#ifndef _PIONEER_MODULE_H
#define _PIONEER_MODULE_H

#include <iri_ros_tools/module.h>
#include <iri_ros_tools/module_service.h>
#include <iri_ros_tools/module_dyn_reconf.h>

#include <pioneer_module/PioneerModuleConfig.h>

//topic
#include <std_msgs/Float64.h>
#include <std_msgs/Bool.h>
//Service
#include <std_srvs/Empty.h>

//States

//Status
typedef enum {PIONEER_MODULE_SUCCESS,
              PIONEER_MODULE_ENABLE_FAILED,
              PIONEER_MODULE_ENABLE_PENDING,
              PIONEER_MODULE_DISABLE_FAILED,
              PIONEER_MODULE_DISABLE_PENDING,
              PIONEER_MODULE_STATE_WATCHDOG,
              PIONEER_MODULE_BATTERY_WATCHDOG,
              PIONEER_MODULE_SET_PARAM_FAIL,
              PIONEER_MODULE_PARAM_NOT_PRESENT} pioneer_module_status_t;

class CPioneerModule : public CModule<pioneer_module::PioneerModuleConfig>
{
  private:

    pioneer_module::PioneerModuleConfig config;

    /****************Services*****************/
    CModuleService<std_srvs::Empty> enable_motors_service;
    CModuleService<std_srvs::Empty> disable_motors_service;
    /****************Services*****************/

    /****************Topics*****************/
    ros::Subscriber battery_voltage_subscriber;
    void battery_voltage_callback(const std_msgs::Float64::ConstPtr& msg);
    CROSWatchdog battery_watchdog;
    double battery_voltage;
    ros::Subscriber motors_state_subscriber;
    void motors_state_callback(const std_msgs::Bool::ConstPtr& msg);
    CROSWatchdog state_watchdog;
    bool motors_enabled;
    /****************Topics*****************/

    CModuleDynReconf pioneer_reconf;

    //Variables
    pioneer_module_status_t status;

  protected:
    void state_machine(void);
    void reconfigure_callback(pioneer_module::PioneerModuleConfig &config, uint32_t level);

  public:
    CPioneerModule(const std::string &name,const std::string &name_space=std::string(""));
    bool enable_motors(void);
    bool disable_motors(void);
    bool are_motors_enabled(bool &enabled);
    bool get_battery_voltage(double &voltage);
    bool set_trans_accel(float accel_m_s_s);
    bool set_trans_decel(float decel_m_s_s);
    bool set_rot_accel(float accel_rad_s);
    bool set_rot_devel(float decel_rad_s);
    bool set_cmd_vel_timeout(float time_s);
    pioneer_module_status_t get_status(void);

    ~CPioneerModule();
  };

  #endif
